@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2012 jeanfi@gmail.com
+ * Copyright (C) 2010-2013 jeanfi@gmail.com
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,9 +21,8 @@
 #include <string.h>
 #include <ctype.h>
 
-#include <gconf/gconf-client.h>
-
 #include "cfg.h"
+#include "log.h"
 
 static const char *KEY_SENSORS = "/apps/psensor/sensors";
 
@@ -34,6 +33,7 @@ static const char *ATT_SENSOR_COLOR = "color";
 static const char *ATT_SENSOR_ENABLED = "enabled";
 static const char *ATT_SENSOR_NAME = "name";
 static const char *ATT_SENSOR_APPINDICATOR_DISABLED = "appindicator/disabled";
+static const char *ATT_SENSOR_POSITION = "position";
 
 static const char *KEY_SENSOR_UPDATE_INTERVAL
 = "/apps/psensor/sensor/update_interval";
@@ -91,6 +91,9 @@ static const char *KEY_INTERFACE_WINDOW_DIVIDER_POS
 static const char *KEY_INTERFACE_TEMPERATURE_UNIT
 = "/apps/psensor/interface/temperature_unit";
 
+static const char *KEY_SLOG_ENABLED = "/apps/psensor/slog/enabled";
+static const char *KEY_SLOG_INTERVAL = "/apps/psensor/slog/interval";
+
 static GConfClient *client;
 
 static char *get_string(const char *key, const char *default_value)
@@ -141,13 +144,9 @@ static struct color *get_foreground_color()
 	return c;
 }
 
-static unsigned int is_alpha_channel_enabled()
+static bool is_alpha_channel_enabled()
 {
-	gboolean b;
-
-	b = gconf_client_get_bool(client, KEY_ALPHA_CHANNEL_ENABLED, NULL);
-
-	return b == TRUE;
+	return gconf_client_get_bool(client, KEY_ALPHA_CHANNEL_ENABLED, NULL);
 }
 
 static enum sensorlist_position get_sensorlist_position()
@@ -306,7 +305,7 @@ config_set_sensor_alarm_low_threshold(const char *sid, int threshold)
 	free(key);
 }
 
-int config_get_sensor_alarm_enabled(const char *sid)
+bool config_get_sensor_alarm_enabled(const char *sid)
 {
 	gboolean b;
 	char *key;
@@ -315,10 +314,10 @@ int config_get_sensor_alarm_enabled(const char *sid)
 	b = gconf_client_get_bool(client, key, NULL);
 	free(key);
 
-	return b == TRUE;
+	return b;
 }
 
-void config_set_sensor_alarm_enabled(const char *sid, int enabled)
+void config_set_sensor_alarm_enabled(const char *sid, bool enabled)
 {
 	char *key;
 
@@ -327,7 +326,7 @@ void config_set_sensor_alarm_enabled(const char *sid, int enabled)
 	free(key);
 }
 
-int config_is_sensor_enabled(const char *sid)
+bool config_is_sensor_enabled(const char *sid)
 {
 	gboolean b;
 	char *key;
@@ -336,10 +335,10 @@ int config_is_sensor_enabled(const char *sid)
 	b = gconf_client_get_bool(client, key, NULL);
 	free(key);
 
-	return b == TRUE;
+	return b;
 }
 
-void config_set_sensor_enabled(const char *sid, int enabled)
+void config_set_sensor_enabled(const char *sid, bool enabled)
 {
 	char *key;
 
@@ -368,7 +367,28 @@ void config_set_sensor_name(const char *sid, const char *name)
 	free(key);
 }
 
-unsigned int config_is_appindicator_enabled(const char *sid)
+int config_get_sensor_position(const char *sid)
+{
+	char *key;
+	int pos;
+
+	key = get_sensor_att_key(sid, ATT_SENSOR_POSITION);
+	pos = gconf_client_get_int(client, key, NULL);
+	free(key);
+
+	return pos;
+}
+
+void config_set_sensor_position(const char *sid, int pos)
+{
+	char *key;
+
+	key = get_sensor_att_key(sid, ATT_SENSOR_POSITION);
+	gconf_client_set_int(client, key, pos, NULL);
+	free(key);
+}
+
+bool config_is_appindicator_enabled(const char *sid)
 {
 	char *key;
 	gboolean b;
@@ -377,69 +397,89 @@ unsigned int config_is_appindicator_enabled(const char *sid)
 	b = gconf_client_get_bool(client, key, NULL);
 	free(key);
 
-	return b == FALSE;
+	return !b;
 }
 
-void config_set_appindicator_enabled(const char *sid, unsigned int enabled)
+void config_set_appindicator_enabled(const char *sid, bool enabled)
 {
 	char *key;
-	gboolean b;
-
-	if (enabled)
-		b = FALSE;
-	else
-		b = TRUE;
 
 	key = get_sensor_att_key(sid, ATT_SENSOR_APPINDICATOR_DISABLED);
-	gconf_client_set_bool(client, key, b, NULL);
+	gconf_client_set_bool(client, key, !enabled, NULL);
 	free(key);
 }
 
-
-static int is_window_decoration_enabled()
+bool is_slog_enabled()
 {
-	gboolean b;
-
-	b = gconf_client_get_bool(client,
-				  KEY_INTERFACE_WINDOW_DECORATION_DISABLED,
-				  NULL);
-
-	return b == FALSE;
+	return gconf_client_get_bool(client, KEY_SLOG_ENABLED, NULL);
 }
 
-static int is_window_keep_below_enabled()
+static void set_slog_enabled(bool enabled)
 {
-	gboolean b;
-
-	b = gconf_client_get_bool(client,
-				  KEY_INTERFACE_WINDOW_KEEP_BELOW_ENABLED,
-				  NULL);
-
-	return b == TRUE;
+	gconf_client_set_bool(client, KEY_SLOG_ENABLED, enabled, NULL);
 }
 
-static void set_window_decoration_enabled(int enabled)
+void config_slog_enabled_notify_add(GConfClientNotifyFunc cbk, void *data)
 {
-	if (enabled)
-		gconf_client_set_bool
-		    (client,
-		     KEY_INTERFACE_WINDOW_DECORATION_DISABLED, FALSE, NULL);
+	log_debug("config_slog_enabled_notify_add");
+	gconf_client_add_dir(client,
+			     KEY_SLOG_ENABLED,
+			     GCONF_CLIENT_PRELOAD_NONE,
+			     NULL);
+	gconf_client_notify_add(client,
+				KEY_SLOG_ENABLED,
+				cbk,
+				data,
+				NULL,
+				NULL);
+}
+
+int config_get_slog_interval()
+{
+	int res;
+
+	res = gconf_client_get_int(client, KEY_SLOG_INTERVAL, NULL);
+
+	if (res <= 0)
+		return 300;
 	else
-		gconf_client_set_bool
-		    (client,
-		     KEY_INTERFACE_WINDOW_DECORATION_DISABLED, TRUE, NULL);
+		return res;
 }
 
-static void set_window_keep_below_enabled(int enabled)
+static void set_slog_interval(int interval)
 {
-	if (enabled)
-		gconf_client_set_bool(client,
-				      KEY_INTERFACE_WINDOW_KEEP_BELOW_ENABLED,
-				      TRUE, NULL);
-	else
-		gconf_client_set_bool(client,
-				      KEY_INTERFACE_WINDOW_KEEP_BELOW_ENABLED,
-				      FALSE, NULL);
+	if (interval <= 0)
+		interval = 300;
+
+	gconf_client_set_int(client, KEY_SLOG_INTERVAL, interval, NULL);
+}
+
+static bool is_window_decoration_enabled()
+{
+	return !gconf_client_get_bool(client,
+				      KEY_INTERFACE_WINDOW_DECORATION_DISABLED,
+				      NULL);
+}
+
+static bool is_window_keep_below_enabled()
+{
+	return gconf_client_get_bool(client,
+				     KEY_INTERFACE_WINDOW_KEEP_BELOW_ENABLED,
+				     NULL);
+}
+
+static void set_window_decoration_enabled(bool enabled)
+{
+	gconf_client_set_bool
+		(client,
+		 KEY_INTERFACE_WINDOW_DECORATION_DISABLED, !enabled, NULL);
+}
+
+static void set_window_keep_below_enabled(bool enabled)
+{
+	gconf_client_set_bool(client,
+			      KEY_INTERFACE_WINDOW_KEEP_BELOW_ENABLED,
+			      enabled, NULL);
 }
 
 /*
@@ -474,6 +514,8 @@ struct config *config_load()
 	c->sensorlist_position = get_sensorlist_position();
 	c->window_decoration_enabled = is_window_decoration_enabled();
 	c->window_keep_below_enabled = is_window_keep_below_enabled();
+	c->slog_enabled = is_slog_enabled();
+	c->slog_interval = config_get_slog_interval();
 
 	c->sensor_update_interval
 	    = gconf_client_get_int(client, KEY_SENSOR_UPDATE_INTERVAL, NULL);
@@ -554,6 +596,8 @@ void config_save(const struct config *c)
 	set_sensorlist_position(c->sensorlist_position);
 	set_window_decoration_enabled(c->window_decoration_enabled);
 	set_window_keep_below_enabled(c->window_keep_below_enabled);
+	set_slog_enabled(c->slog_enabled);
+	set_slog_interval(c->slog_interval);
 
 	gconf_client_set_int(client,
 			     KEY_GRAPH_UPDATE_INTERVAL,
