@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2012 jeanfi@gmail.com
+ * Copyright (C) 2010-2013 jeanfi@gmail.com
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -17,10 +17,15 @@
  * 02110-1301 USA
  */
 #include <stdlib.h>
+#include <string.h>
+
 #include <sys/time.h>
+
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
 #include "cfg.h"
+#include "log.h"
 #include "psensor.h"
 
 /* horizontal padding */
@@ -211,6 +216,23 @@ static void draw_sensor_curve(struct psensor *s,
 	cairo_stroke(cr);
 }
 
+static void display_no_graphs_warning(cairo_t *cr, int x, int y)
+{
+	char *msg;
+
+	msg = strdup(_("No graphs enabled"));
+
+	cairo_select_font_face(cr,
+			       "sans-serif",
+			       CAIRO_FONT_SLANT_NORMAL,
+			       CAIRO_FONT_WEIGHT_NORMAL);
+	cairo_set_font_size(cr, 18.0);
+
+	cairo_move_to(cr, x, y);
+	cairo_show_text(cr, msg);
+
+	free(msg);
+}
 
 void
 graph_update(struct psensor **sensors,
@@ -223,12 +245,12 @@ graph_update(struct psensor **sensors,
 	double min_rpm, max_rpm, mint, maxt;
 	char *strmin, *strmax;
 	/* horizontal and vertical offset of the graph */
-	int g_xoff, g_yoff;
+	int g_xoff, g_yoff, no_graphs, min, max;
 	cairo_surface_t *cst;
 	cairo_t *cr, *cr_pixmap;
 	char *str_btime, *str_etime;
 	cairo_text_extents_t te_btime, te_etime, te_max, te_min;
-	struct psensor **sensor_cur;
+	struct psensor **sensor_cur, **enabled_sensors;
 	GtkAllocation galloc;
 	GtkStyleContext *style_ctx;
 	GdkRGBA rgba;
@@ -236,16 +258,17 @@ graph_update(struct psensor **sensors,
 	if (!gtk_widget_is_drawable(w_graph))
 		return ;
 
-	min_rpm = get_min_rpm(sensors);
-	max_rpm = get_max_rpm(sensors);
+	enabled_sensors = psensor_list_filter_graph_enabled(sensors);
 
-	mint = get_min_temp(sensors);
+	min_rpm = get_min_rpm(enabled_sensors);
+	max_rpm = get_max_rpm(enabled_sensors);
 
+	mint = get_min_temp(enabled_sensors);
 	strmin = psensor_value_to_str(SENSOR_TYPE_TEMP,
 				      mint,
 				      config->temperature_unit == CELCIUS);
 
-	maxt = get_max_temp(sensors);
+	maxt = get_max_temp(enabled_sensors);
 	strmax = psensor_value_to_str(SENSOR_TYPE_TEMP,
 				      maxt,
 				      config->temperature_unit == CELCIUS);
@@ -329,38 +352,40 @@ graph_update(struct psensor **sensors,
 	et = get_graph_end_time_s();
 
 	if (bt && et) {
-		sensor_cur = sensors;
+		sensor_cur = enabled_sensors;
 
 		cairo_set_line_join(cr, CAIRO_LINE_JOIN_ROUND);
 		cairo_set_line_width(cr, 1);
+		no_graphs = 1;
 		while (*sensor_cur) {
 			struct psensor *s = *sensor_cur;
 
-			if (s->enabled) {
-				double min, max;
-
-				if (is_fan_type(s->type)) {
-					min = min_rpm;
-					max = max_rpm;
-				} else if (s->type & SENSOR_TYPE_CPU_USAGE) {
-					min = 0;
-					max = get_max_value
-						(sensors,
-						 SENSOR_TYPE_CPU_USAGE);
-				} else {
-					min = mint;
-					max = maxt;
-				}
-
-				draw_sensor_curve(s, cr,
-						  min, max,
-						  bt, et,
-						  g_width, g_height,
-						  g_xoff, g_yoff);
+			no_graphs = 0;
+			if (is_fan_type(s->type)) {
+				min = min_rpm;
+				max = max_rpm;
+			} else if (s->type & SENSOR_TYPE_CPU_USAGE) {
+				min = 0;
+				max = get_max_value(enabled_sensors,
+						    SENSOR_TYPE_CPU_USAGE);
+			} else {
+				min = mint;
+				max = maxt;
 			}
 
+			draw_sensor_curve(s, cr,
+					  min, max,
+					  bt, et,
+					  g_width, g_height,
+					  g_xoff, g_yoff);
+
 			sensor_cur++;
 		}
+
+		if (no_graphs)
+			display_no_graphs_warning(cr,
+						  g_xoff + 12,
+						  g_height / 2);
 	}
 
 	cr_pixmap = gdk_cairo_create(gtk_widget_get_window(w_graph));
@@ -374,6 +399,8 @@ graph_update(struct psensor **sensors,
 		cairo_paint(cr_pixmap);
 	}
 
+	free(enabled_sensors);
+
 	cairo_destroy(cr_pixmap);
 	cairo_surface_destroy(cst);
 	cairo_destroy(cr);
