@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2012 jeanfi@gmail.com
+ * Copyright (C) 2010-2013 jeanfi@gmail.com
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,36 +19,44 @@
 #ifndef _PSENSOR_PSENSOR_H_
 #define _PSENSOR_PSENSOR_H_
 
-#include "config.h"
 #include <sensors/sensors.h>
 
+#include "config.h"
 #ifdef HAVE_ATASMART
 #include <atasmart.h>
 #endif
 
+#include "bool.h"
 #include "color.h"
 #include "log.h"
 #include "measure.h"
 
 enum psensor_type {
-	SENSOR_TYPE_TEMP = 0x0001,
-	SENSOR_TYPE_FAN = 0x0002,
-	SENSOR_TYPE_REMOTE = 0x0004,
+	/* type of sensor values */
+	SENSOR_TYPE_TEMP = 0x00001,
+	SENSOR_TYPE_RPM = 0x00002,
+	SENSOR_TYPE_USAGE = 0x00004,
 
-	SENSOR_TYPE_LMSENSOR = 0x0100,
-	SENSOR_TYPE_NVIDIA_TEMP = 0x0200 | SENSOR_TYPE_TEMP,
-	SENSOR_TYPE_HDD_TEMP = 0x0400 | SENSOR_TYPE_TEMP,
-	SENSOR_TYPE_CPU_USAGE = 0x0800,
-	SENSOR_TYPE_AMD = 0x1000,
+	/* Whether the sensor is remote */
+	SENSOR_TYPE_REMOTE = 0x00008,
 
-	SENSOR_TYPE_HDD_TEMP_ATASMART = 0x2000 | SENSOR_TYPE_HDD_TEMP,
-	SENSOR_TYPE_HDD_TEMP_HDDTEMP = 0x4000 | SENSOR_TYPE_HDD_TEMP,
+	/* Libraries used for retrieving sensor information */
+	SENSOR_TYPE_LMSENSOR = 0x00100,
+	SENSOR_TYPE_NVCTRL = 0x00200,
+	SENSOR_TYPE_GTOP = 0x00400,
+	SENSOR_TYPE_ATIADL = 0x00800,
+	SENSOR_TYPE_ATASMART = 0x01000,
+	SENSOR_TYPE_HDDTEMP = 0x02000,
 
-	SENSOR_TYPE_AMD_TEMP = SENSOR_TYPE_AMD | SENSOR_TYPE_TEMP,
-	SENSOR_TYPE_AMD_FAN = SENSOR_TYPE_AMD | SENSOR_TYPE_FAN,
+	/* Type of HW component */
+	SENSOR_TYPE_HDD = 0x04000,
+	SENSOR_TYPE_CPU = 0x08000,
+	SENSOR_TYPE_GPU = 0x10000,
+	SENSOR_TYPE_FAN = 0x20000,
 
-	SENSOR_TYPE_LMSENSOR_TEMP = SENSOR_TYPE_LMSENSOR | SENSOR_TYPE_TEMP,
-	SENSOR_TYPE_LMSENSOR_FAN = SENSOR_TYPE_LMSENSOR | SENSOR_TYPE_FAN
+	/* Combinations */
+	SENSOR_TYPE_HDD_TEMP = (SENSOR_TYPE_HDD | SENSOR_TYPE_TEMP),
+	SENSOR_TYPE_CPU_USAGE = (SENSOR_TYPE_CPU | SENSOR_TYPE_USAGE)
 };
 
 struct psensor {
@@ -58,6 +66,9 @@ struct psensor {
 	/* Uniq id of the sensor */
 	char *id;
 
+	/* Name of the chip. */
+	char *chip;
+
 	/* lm-sensor */
 	const sensors_chip_name *iname;
 	const sensors_feature *feature;
@@ -73,7 +84,7 @@ struct psensor {
 	struct color *color;
 
 	/* Whether the graph sensor is displayed. */
-	unsigned int enabled;
+	bool graph_enabled;
 
 	/* see psensor_type */
 	unsigned int type;
@@ -85,7 +96,7 @@ struct psensor {
 	double min;
 
 	/* Whether alarm alert is enabled for this sensor */
-	unsigned int alarm_enabled;
+	bool alarm_enabled;
 
 	int alarm_high_threshold;
 	int alarm_low_threshold;
@@ -107,14 +118,15 @@ struct psensor {
 #ifdef HAVE_ATASMART
 	SkDisk *disk;
 #endif
-
 	char *url;
 
-	unsigned int appindicator_enabled;
+	bool appindicator_enabled;
 };
 
 struct psensor *psensor_create(char *id,
-			       char *name, unsigned int type,
+			       char *name,
+			       char *chip,
+			       unsigned int type,
 			       int values_max_length);
 
 void psensor_values_resize(struct psensor *s, int new_size);
@@ -124,6 +136,8 @@ void psensor_free(struct psensor *sensor);
 void psensor_list_free(struct psensor **sensors);
 int psensor_list_size(struct psensor **sensors);
 
+struct psensor **psensor_list_filter_graph_enabled(struct psensor **);
+
 struct psensor *psensor_list_get_by_id(struct psensor **sensors,
 				       const char *id);
 
@@ -166,11 +180,13 @@ struct psensor **get_all_sensors(int use_libatasmart, int values_max_length);
 struct psensor **psensor_list_add(struct psensor **sensors,
 				  struct psensor *sensor);
 
+struct psensor **psensor_list_copy(struct psensor **);
+
 void psensor_set_current_value(struct psensor *sensor, double value);
 void psensor_set_current_measure(struct psensor *sensor, double value,
 				 struct timeval tv);
 
-double psensor_get_current_value(struct psensor *sensor);
+double psensor_get_current_value(const struct psensor *);
 
 struct measure *psensor_get_current_measure(struct psensor *sensor);
 
@@ -192,6 +208,7 @@ double get_max_value(struct psensor **sensors, int type);
 double celcius_to_fahrenheit(double c);
 double fahrenheit_to_celcius(double c);
 
+char *psensor_current_value_to_str(const struct psensor *, unsigned int);
 
 void psensor_log_measures(struct psensor **sensors);
 
