@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2012 jeanfi@gmail.com
+ * Copyright (C) 2010-2013 jeanfi@gmail.com
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,19 +19,26 @@
 #ifndef _PSENSOR_SERVER_H_
 #define _PSENSOR_SERVER_H_
 
-#include "psensor.h"
-#include "sysinfo.h"
+#include "config.h"
 
-#define URL_BASE_API_1_0 "/api/1.0"
-#define URL_BASE_API_1_0_SENSORS "/api/1.0/sensors"
-#define URL_API_1_0_SERVER_STOP "/api/1.0/server/stop"
-#define URL_API_1_0_SYSINFO "/api/1.0/sysinfo"
-#define URL_API_1_0_CPU_USAGE "/api/1.0/cpu/usage"
+#include "psensor.h"
+
+#ifdef HAVE_GTOP
+#include "sysinfo.h"
+#endif
+
+#define URL_BASE_API_1_1 "/api/1.1"
+#define URL_BASE_API_1_1_SENSORS "/api/1.1/sensors"
+#define URL_API_1_1_SERVER_STOP "/api/1.1/server/stop"
+#define URL_API_1_1_SYSINFO "/api/1.1/sysinfo"
+#define URL_API_1_1_CPU_USAGE "/api/1.1/cpu/usage"
 
 struct server_data {
 	struct psensor *cpu_usage;
 	struct psensor **sensors;
+#ifdef HAVE_GTOP
 	struct psysinfo psysinfo;
+#endif
 	char *www_dir;
 };
 
