@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2012 jeanfi@gmail.com
+ * Copyright (C) 2010-2013 jeanfi@gmail.com
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,6 +21,8 @@
 
 #include "config.h"
 
+#include <pthread.h>
+
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
@@ -34,17 +36,19 @@
 
 struct ui_psensor {
 	struct psensor **sensors;
+	/* mutex which MUST be used for accessing sensors.*/
+	pthread_mutex_t sensors_mutex;
 
 	GtkWidget *w_graph;
 
-	struct ui_sensorlist *ui_sensorlist;
-
 	struct config *config;
 
 	GtkWidget *main_window;
 
 	GtkWidget *menu_bar;
 
+	GtkWidget *popup_menu;
+
 	/*
 	 * The main vertical box, top contains the menubar, bottom
 	 * contains the sensor_box.
@@ -55,11 +59,13 @@ struct ui_psensor {
 	 * The box which contains the sensors graph and the sensors
 	 * information list.
 	 */
-	GtkWidget *sensor_box;
+	GtkPaned *sensor_box;
+
+	GtkListStore *sensors_store;
+	GtkTreeView *sensors_tree;
+	GtkScrolledWindow *sensors_scrolled_tree;
 
 	int graph_update_interval;
-
-	GMutex *sensors_mutex;
 };
 
 /*
@@ -87,4 +93,10 @@ void ui_show_about_dialog();
 
 void ui_enable_alpha_channel(struct ui_psensor *ui);
 
+void ui_cb_preferences(GtkMenuItem *mi, gpointer data);
+void ui_cb_about(GtkMenuItem *mi, gpointer data);
+void ui_cb_menu_quit(GtkMenuItem *mi, gpointer data);
+void ui_cb_sensor_preferences(GtkMenuItem *mi, gpointer data);
+
+struct psensor **ui_get_sensors_ordered_by_position(const struct ui_psensor *);
 #endif
